Struct sfml::graphics::RenderTexture [] [src]

pub struct RenderTexture {
    // some fields omitted
}

Target for off-screen 2D rendering into a texture

Methods

impl RenderTexture
[src]

fn new(width: u32, height: u32, depth_buffer: bool) -> Option<RenderTexture>

Construct a new render texture

Arguments

  • width - Width of the render texture
  • height - Height of the render texture
  • depthBuffer - Do you want a depth-buffer attached? (useful only if you're doing 3D OpenGL on the rendertexture)

Return Some(RenderTexture) or None

fn display(&self)

Update the contents of the target texture

fn set_active(&mut self, active: bool) -> bool

Activate or deactivate a render texture as the current target for rendering

Arguments

  • active - true to activate, false to deactivate

fn get_texture(&self) -> Option<Texture>

Get the target texture of a render texture

Return the target texture

fn set_smooth(&mut self, smooth: bool)

Enable or disable the smooth filter on a render texture

Arguments

  • smooth - true to enable smoothing, false to disable it

fn is_smooth(&self) -> bool

Tell whether the smooth filter is enabled or not for a render texture

Return true if smoothing is enabled, false if it is disabled